#include <iostream>
#include <SFML/Graphics.hpp>
#include <julia.h>

int main()
{
    sf::VideoMode vmode = sf::VideoMode::getDesktopMode();
    sf::RenderWindow window(vmode,"Julia Set", sf::Style::Fullscreen);
    sf::Event e;
    sf::RenderStates states;

    sf::RectangleShape rect;
    rect.setSize(sf::Vector2f(window.getSize()));
    rect.setFillColor(sf::Color::Black);

    sf::Shader juliaShader;
    juliaShader.loadFromMemory(julia, sf::Shader::Fragment);
    juliaShader.setParameter("resolution", (sf::Vector2f)window.getSize());

    sf::Clock clock;
    sf::Time frameTime = sf::Time::Zero;
    sf::Time timePerFrame = sf::seconds(1/60.f);
    sf::Time dt = sf::Time::Zero;

    sf::Vector2i mousePos;
    sf::Vector2i currPos = sf::Vector2i(0,0);
    sf::Mouse::setPosition(sf::Vector2i(0,0));

    for(;;)
    {
        while(window.pollEvent(e))
        {
            if(e.type == sf::Event::Closed)
            {
                window.close();
                return 0;
            }

            if(e.type == sf::Event::KeyPressed)
            {
                switch(e.key.code)
                {
                case sf::Keyboard::Escape:
                    {
                        window.close();
                        return 0;
                    }
                    break;
                default:
                    break;
                }
            }

            mousePos = sf::Mouse::getPosition(window);

            if(abs(mousePos.x - currPos.x) > 15
               || abs(mousePos.y - currPos.x) > 15)
            {
                window.close();
                return 0;
            }
        }

        frameTime += clock.restart();
        while(frameTime > timePerFrame)
        {
            frameTime -= timePerFrame;

            //reset the animation after 5 min...it get's strange
            //when the shader runs too long
            if(dt.asSeconds() > 300)
                dt = sf::Time::Zero;
            else
                dt += timePerFrame;

            juliaShader.setParameter("time", dt.asSeconds());
        }

        states.shader = &juliaShader;

        window.clear(sf::Color(200,100,0,255));
        window.draw(rect, states);
        window.display();
    }
    return 0;
}
